"""Object Detection
"""
__all__ = ['jsonfile', 'bbox', 'video']


import os
import shutil
import time
import numpy as np
import json
from functools import partial
from datetime import datetime
from PIL import Image, ImageDraw, ImageFont
from .utils import draw_bbox, load_graph, run_inference_for_video, object_to_json_style


def jsonfile(json_data, output_filename):
    """json string to json file
    """
    json_data = json.dumps(json_data, ensure_ascii=False, indent=4)
    dir_path = os.path.dirname(output_filename)

    if dir_path and not os.path.isdir(dir_path):
        os.makedirs(os.path.dirname(output_filename))

    with open(output_filename, 'w') as f:
        f.write(json_data)


def bbox(video_objects:list, np_images:np.array, testing_mode=False)->list:
    """numpy array 형태의 이미지와 bbox를 그릴 수 있도록 정리된 object들을 주면 
       image에 그려서 리턴한다.
    """
    drawn_images = []
    for image_objects, np_image in zip(video_objects, np_images):
        im = Image.fromarray(np_image)
        draw = ImageDraw.Draw(im)
        for image_object in image_objects:
            drawn_bbox = draw_bbox(image_object, draw, testing_mode)
            drawn_bbox()
        drawn_images.append(im)
    return drawn_images


def video(file_name, PATH_TO_FROZEN_GRAPH, label_map, score_threshold=0.7, 
            gpu_num=None, allow_growth=True, process_gpu_memory=5000):
    """ Object Detection 및 결과 정리
    """
    video_objects = []
    # load graph and detection
    detection_graph = load_graph(PATH_TO_FROZEN_GRAPH)
    analized_images, np_images = run_inference_for_video(file_name, detection_graph, gpu_num, 
                                                            allow_growth, process_gpu_memory)

    # 가장 바깥 for문은 비디오안에 이미지들을 반복한다.
    # 다음 for문은 이미지안에 오브젝트들을 반복한다.
    for analized_image, np_image in zip(analized_images, np_images):
        image_objects = []
        for score, object_num, box in zip(
            analized_image["detection_scores"],
            analized_image["detection_classes"],
            analized_image["detection_boxes"],):
            if score > score_threshold:
                image_objects.append(
                    object_to_json_style(
                        object_num, score, box, np_image, label_map))
        video_objects.append(image_objects)
    return video_objects, np_images


if __name__ == "__main__":
    # if __package__ is None:
    #     import sys
    #     from os import path
    #     print(path.dirname(path.dirname(path.abspath(__file__))))
    #     sys.path.append(path.dirname(path.dirname(path.abspath(__file__))))
    #     from .utils import draw_bbox, load_graph, run_inference_for_video, object_to_json_style
    # else:
    #     from .utils import draw_bbox, load_graph, run_inference_for_video, object_to_json_style
    #     from ..utils.image_to_video import image_to_video_array
    video_objects, np_images = video(
        "/Users/sangpil/python_project/medi-package/medi_7.mp4",
        "/Users/sangpil/python_project/medi-package/test.pb",
        "/Users/sangpil/python_project/medi-package/label_map.pbtxt"
        )

    drawn_images = bbox(video_objects, np_images)
    # image_to_video_array("test.mp4",drawn_images)