import re

from functools import lru_cache
from google.protobuf import text_format


from ..protos import label_map_pb2

class ObjectList:

    @lru_cache()
    def __init__(self, label_map):
        with open(label_map, 'r') as f:
            label_map = label_map_pb2.LabelMap()
            text_format.Merge(f.read(), label_map)
            self.objs = dict((i.name, 0) for i in label_map.item)

    def __repr__(self):
        return str(self.objs)

    def __setitem__(self, key, value):
        if key in self.objs.keys() and type(value) is int:
            self.objs[key] = value

    def __getitem__(self, item):
        return self.objs[item]

    def __len__(self):
        return len(self.objs)

    def make_feature(self):
        return [obj for obj in self.objs.values()]

    def item_list(self):
        return [key for key in self.objs.keys()]

    def item_group_list(self, category):
        return [obj_name for obj_name in self.item_list() if
                re.compile(f'({category})+$').search(obj_name)]


class Medication(ObjectList):
    def __init__(self, degree: int, label_map):
        super().__init__()