__all__ = ['AzureBlob']

import os
import time
import logging
from typing import Dict, List, Tuple, Any
import os

from django.conf import settings
from django.core.cache import cache

from azure.storage.blob import PublicAccess, BlobServiceClient

from medi.utils.deco import get_time
from medi.object_detection.utils.object_detection_utils import videoframe
logger = logging.getLogger("medication.download")

class AzureBlob:
    def __new__(cls, *args, **kwargs):
        if len(args) < 2:
            return None
        return object.__new__(cls)
    
    def __init__(self, connect_str: str, container_name: str) -> None:
        
        self.__connect_str = connect_str
        self.container_name = container_name
        self.blob_service_client = BlobServiceClient.from_connection_string(connect_str)
        self.container_client = self.blob_service_client.get_container_client(container_name)
        
    def __repr__(self) -> str:
        return f"container_name: {self.container_name}"

    def videos(self) -> List[Dict[str, Any]]:
        """
        Make Azure Blob videos list
        Ordered by uploaded time
        """
        video_list = list(
            [
                {"name": i.name, "time": i.creation_time}
                for i in self.container_client.list_blobs()
            ]
        )
        video_list = sorted(
            video_list, key=lambda video_list: video_list["time"], reverse=True
        )

        video_list = list(i["name"] for i in video_list)

        cache.get_or_set("video_list", video_list)
        return video_list

    @get_time
    def download(self, filename: str) -> bool:
        """
        download file
        return (bool)
        """
        blob_list: list
        now = time.time()
        download_file_path = os.path.join(settings.VIDEO_PATH, filename)
        
        blob_content = self.blob_service_client.get_blob_client(
            container=self.container_name, blob=filename
        )
        blob_list = self.videos()

        if os.path.isfile(download_file_path):
            touch_result = os.system(f'touch {download_file_path}')
            logger.debug(f'touch {download_file_path} | {touch_result}')
            return True

        if filename in blob_list:
            if not os.path.exists(os.path.dirname(download_file_path)):
                os.makedirs(os.path.dirname(download_file_path))
            with open(download_file_path, "wb") as download_file:
                download_file.write(blob_content.download_blob().readall())
            return True
        else:
            print("file is not exist!!!!!!!!!")
            return False
